use Test::More;
use strict;
use Font::TTF::Font;
use Font::EOTWrapper;
use Font::FreeType;
use Font::TypefaceJS;
use File::Compare qw ( compare );

plan tests => 10;

my $SourceDir = "$ENV{EPAGES_CARTRIDGES}/DE_EPAGES/Core/Data/Fonts";

my %Expected = (
    'DejaVuSerif-Bold' => {
        family_name => 'DejaVu Serif',
        style_name => 'Bold',
        postscript_name => 'DejaVuSerif-Bold',
    },
    'LiberationMono-Italic' => {
        family_name => 'Liberation Mono',
        style_name => 'Italic',
        postscript_name => 'LiberationMono-Italic',
    },
    );

foreach my $Font (keys %Expected) {
    my $Face = Font::FreeType->new->face("$SourceDir/$Font.ttf");
    ok($Face->$_ eq $Expected{$Font}{$_}, $Face->$_. " eq $Expected{$Font}{$_}")
        foreach keys %{$Expected{$Font}};

    my $Typeface = Font::TypefaceJS->new(input_filename => "$SourceDir/$Font.ttf");
    $Typeface->write_file(output_filename => "$Font.js");
    Font::EOTWrapper::convert("$SourceDir/$Font.ttf", "$Font.eot");

    foreach my $Code (qw(js eot)) {
        my $Result = compare("$Font.$Code", "$Code/$Font.$Code");
        ok($Result == 0, "compare('$Font.$Code', '$Code/$Font.$Code') == $Result");
        unlink("$Font.$Code") if $Result == 0;
    }
}
