use strict;
use Test::Simple tests => 11;
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::CrossSelling::API::CrossSelling qw( ExistsCrossSelling FindCrossSelling );
use DE_EPAGES::Object::API::Factory qw( ExistsObjectByPath LoadObjectByPath LoadRootObject );
use DE_EPAGES::Database::API::Connection qw( ConnectTestSite DisconnectSite );

my $ATTRIBUTE_PACKAGE = 'DE_EPAGES::Object::API::Attributes::DefaultAttribute';
my $ROOTPATH = '/Test';

sub Main {
    ConnectTestSite();

    # create test classes
    ok( Init(), "initialization" );

    my $ParentObject = LoadObjectByPath( $ROOTPATH );
    my $Shop = $ParentObject->insertChild( 'Shop', { Alias => 'TestShop' } );
    ok( $Shop->exists, "insert TestShop" );

    $Shop->featureMaxValue( 'Products', 10 );

    my $Products = $Shop->folder('Products');
    my $Product1 = $Products->insertChild( 'Product', {
        Alias => 'Product1',
        Shop => $Shop,
    } );
    ok( $Product1->exists, "insert Product1" );
    my $Product2= $Products->insertChild( 'Product', {
        Alias => 'Product2',
        Shop => $Shop,
    } );
    ok( $Product2->exists, "insert Product2" );
    my $Product3= $Products->insertChild( 'Product', {
        Alias => 'Product3',
        Shop => $Shop,
    } );
    ok( $Product3->exists, "insert Product3" );

    ok (! ExistsCrossSelling($Product1, $Product2, 'CrossSelling'), 'Product2 is not assigned to Product1');
    ok (! ExistsCrossSelling($Product1, $Product3, 'CrossSelling'), 'Product3 is not assigned to Product1');
    ok (! defined FindCrossSelling($Product1, $Product2, 'CrossSelling'), 'CrossSelling is not defined for Product1 with Product2');

    my $FolderCrossSelling = $Product1->folder('CrossSelling');
    my $CrossSelling = $FolderCrossSelling->insertChild( 'CrossSelling', {
        Alias => $FolderCrossSelling->nextAlias,
        Product => $Product1,
        TargetProduct => $Product2,
    });

    ok (ExistsCrossSelling($Product1, $Product2, 'CrossSelling'), 'Product2 is assigned to Product1');
    ok (FindCrossSelling($Product1, $Product2, 'CrossSelling')->id == $CrossSelling->id, 'CrossSelling for Product1 and Product2 found');

    ok( Cleanup(), 'Cleanup' );
    DisconnectSite();
}

sub Cleanup {
    if( ExistsObjectByPath( $ROOTPATH ) ) {
        LoadObjectByPath( $ROOTPATH )->delete;
    }
    return 1;
}

sub Init {
    Cleanup();

    LoadRootObject()->insertChild( 'Object', { Alias => 'Test' });
    return 1;
}

RunScript(Sub=>\&Main, Debug=>1);
