#=====================================================================================
# Searches for indexes where the name does not match the naming convention /^[ui]_/.
# If the index name starts with 'fk_' it has been implicitely created by MySQL and may
# cause trouble when restoring the database from a dump (see EPG-20285 for details).
#=====================================================================================

use strict;

use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::Database::API::Connection qw ( GetCurrentDBHandle GetTestSite RunOnStore );

use Test::More tests => 1;

use constant STMT => <<SQL ;
SELECT  s.INDEX_NAME, s.TABLE_NAME, s.COLUMN_NAME
FROM information_schema.STATISTICS s
WHERE TABLE_schema = ?
AND NOT (index_name = "PRIMARY"
         OR index_name LIKE "i_%"
         OR index_name LIKE "u_%");
SQL

sub Test {

    my $dbi = GetCurrentDBHandle();

    SKIP: {
        skip "No MySQL DBD driver\n", 1 if $dbi->driverName ne 'mysql';

        my $aaResult = $dbi->execute( STMT, [ $dbi->databaseName ] );
        my $msg = '';
        foreach my $Rows (@$aaResult) {
            $msg .= join( ', ', @$Rows ) . "\n";
        }
        my $heading =
        'Found ('
        . scalar @$aaResult
        . ") illegal indexes (see EPG-20285):\nIndex Name, Table Name, Column Name\n";

        ok( !$msg, 'mysql uses only explicit modeled indexes' ) or diag( $heading . $msg );

    }
}

RunScript(
    Debug => 1,
    Sub   => sub {
        RunOnStore(
            Store => GetTestSite,
            Sub   => sub {
                Test();
            }
        );
    }
);
