use strict;

use DE_EPAGES::Core::API::Error qw ( ExistsError GetError );
use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::Database::API::Connection qw ( GetTestSite RunOnStore );
use DE_EPAGES::DocdataPay::API::Object::RemoteDocdataPay;
use DE_EPAGES::DocdataPay::t::MockUserAgentStatus;
use DE_EPAGES::DocdataPay::t::MockUserAgentXML;
use Test::More tests => 3;

sub testTransaction {
    my $Remote = DE_EPAGES::DocdataPay::API::Object::RemoteDocdataPay->new(
        'ServiceURL' => 'https://test.docdatapayments.com/ps/com.tripledeal.paymentservice.servlets.PaymentService',
        'IsLogging'  => 1,
    );

    # EPG-21713: mock the web request to avoiding service failures 
    *LWP::UserAgent::post = sub {
        return DE_EPAGES::DocdataPay::t::MockUserAgentXML->new();
    };
    $Remote->newPaymentCluster( {
            'merchant_name'           => 'epages_com',
            'merchant_password'       => 'AbG2987',
            'merchant_transaction_id' => time(),
            'profile'                 => 'standard',
            'client_id'               => '1001',
            'price'                   => 102.15,
            'cur_price'               => 'EUR',
            'client_email'            => 'o.trosien@epages.com',
            'client_firstname'        => 'Oliver',
            'client_lastname'         => 'Trosien',
            'client_address'          => 'Pilatuspool 2',
            'client_zip'              => '20355',
            'client_city'             => 'Hamburg',
            'client_country'          => 'DE',
            'client_language'         => 'en',
            'description'             => '1x Campingaz Trister',
            'days_pay_period'         => 14
        }
    );

    ok( defined $Remote->paymentClusterID(),  'transaction id: ' . $Remote->paymentClusterID() );
    ok( defined $Remote->paymentClusterKey(), 'transaction key: ' . $Remote->paymentClusterKey() );

    # EPG-21713: mock the web request to avoiding service failures
    *LWP::UserAgent::post = sub {
        return DE_EPAGES::DocdataPay::t::MockUserAgentStatus->new();
    };
    my $TransStatus = $Remote->statusPaymentCluster( {
            'payment_cluster_key' => $Remote->paymentClusterKey(),
            'merchant_name'       => 'epages_com',
            'merchant_password'   => 'AbG2987',
        }
    );
    is( $TransStatus, 'NY', 'Status: ' . $TransStatus );
}

sub Main {
    RunOnStore(
        'Store' => GetTestSite(),
        'Sub'   => sub {
            eval { testTransaction; };
            if ( ExistsError() ) {
                my $Error = GetError();
                fail( $Error->longMessage );
            }
          }
    );

    done_testing();
}

RunScript( Sub => \&Main );

1;
