/*
 * @copyright   © Copyright 2006-2012, epages GmbH, All Rights Reserved.
 *
 * @module      ep.canvas
 *
 * @base      http://canvas.quaese.de/
 *
 * @revision    $Revision: 1.3.6.4 $
 */

 (function ($){

	var dependency = ['ep.core'];

//#JSCOVERAGE_IF $.browser.msie
	if ($.browser.msie && parseInt($.browser.version, 10) < 9){
	dependency.push('jQuery.canvas.excanvas');
	dependency.push('jQuery.canvas.canvastext');
	}
//#JSCOVERAGE_ENDIF


	$.plugin( 'ep.canvas', {
		plugin: dependency
	}, function( $, undefined ){

		var init = function(canvas){
			//if the context feature don't exist and the IE CanvasManager-Object is available
			if((!canvas.getContext) && (G_vmlCanvasManager !== undefined)){
				canvas = G_vmlCanvasManager.initElement(canvas);
			}

			// if the context feature exists
			if(canvas.getContext){
				canvas.ctx = canvas.getContext('2d');
				return canvas;
			}

			return null;
		},

		degToRad = function(angle){
			return angle * Math.PI/180;
		},

		calcDimensions = function(lineHeight, defaults, ctx){
			var multiLines = defaults.text.split(defaults.delimiter),
					multiObj = {},
					measureWidth = 0,
					newWidth, newHeight;

			// determine the max line width
			for(var i=0, iLen=multiLines.length; i<iLen; i++){
				multiObj[i] = {};
				multiObj[i].width = ctx.measureText(multiLines[i]).width;
				measureWidth = (multiObj[i].width > measureWidth)? multiObj[i].width : measureWidth;
			}

			var arcTan = Math.atan(measureWidth/(lineHeight*multiLines.length)),
					diagonal = Math.sqrt(measureWidth*measureWidth + lineHeight*multiLines.length*lineHeight*multiLines.length),
					newAngle = defaults.phi,
					argument = {};

			if(newAngle > 90 && newAngle < 180 || newAngle > 270 && newAngle < 360){
				argument.x = degToRad(newAngle) - arcTan - degToRad(90);
				argument.y = degToRad(180) - degToRad(newAngle) - arcTan ;
			}else{
				argument.x = degToRad(90) - degToRad(newAngle) - arcTan;
				argument.y = arcTan - degToRad(newAngle);
			}

			newWidth = Math.abs(Math.cos(argument.x) * diagonal);
			newHeight = Math.abs(Math.cos(argument.y) * diagonal);

			return {w: newWidth, h: newHeight, measureWidth: measureWidth, multiLines: multiLines, lineHeight: lineHeight};
		},

		qpCanvas = function(){},
		fn = qpCanvas.prototype;

		// extend qpCanvas-prototype with own attributes and methods
		// @ this   = canvas-element (collection)
		// @ this.ctx = context (getContext('2d') (collection)
		// @ this[0]  = first canvas-element
		// @ this[0]  = first context (getContext('2d'))
		$.extend(fn, {
			length: 0,                      // length property
			push: Array.prototype.push,     // inherit push-method from Array-Object
			each: jQuery.fn.each,           // inherit each-method from JQuery-Object

			end: function(){                // simulate jQuery end-method
				return this.prevObject || $();
			},

			// calculates the dimensions of an multiline text
			// returns an object, containing some useful sizes/dimensions as
			// - w: (float) new width of surrounding canvas,
			// - h: (float) new height of surrounding canvas,
			// - innerW: (float) new width of text element,
			// - innerH: (float) new height of text element,
			// - measureWidth: (float) new width of text element,
			// - multiLines: (array) splitted text,
			// - lineHeight: (float) height of a single line,
			// - scale: (float) scale factor (can be set to a differ value from 1 when resizableCanvas is set to false),
			// - text: (string) text that fits into the maxHeight and maxWidth dimensions,
			// - x: (float) horizontal offset of multiline text in canvas element,
			// - y: (float) vertical offset of multiline text in canvas element
			rescaleDimensions: function(options){
				var defaults = $.extend({
					x: 0,                   // horizontal coordinate
					y: 0,                   // vertical coordinate
					style: 'fillText',      // text method (fillText or strokeText)
					fontSize: 12,           // font size
					fontFamily: 'Arial',    // font family
					fontUnit: 'pt',         // font unit
					phi: 0,                 // rotate angle (in degree)
					delimiter: "\n",        // delimter t o detect new lines
					align: "left",          // text align (left, right, center)
					rp: 'c',                // rotation coordinates: nw, n, ne, w, c (default), e, sw, s, se
					correctLineHeight: 0,   // useful to adjust the lineheight
					minSize: 10,            // minimal size a text can be scaled
					maxWidth: undefined,    // if set to an number, the multitext dimensions will be calculated depending to its borders
					maxHeight: undefined,   // if set to an number, the multitext dimensions will be calculated depending to its borders
					resizeCanvas: false,    // if set to true, the multiline will be trimmed so that it fits into the borders (maxWidth and/or maxHeight)
					resizable: true,        // if set to true font size will be scaled when size of multiline is greater than maxWidth and/or maxHeight and fit into the borders
					text: 'dummy\ndummy'
				}, options);

				// test text method
				defaults.style = (defaults.style.match(/fillText|strokeText/))? defaults.style : 'fillText';

				// set font
				this[0].ctx.save();
				this[0].ctx.font = defaults.fontSize + defaults.fontUnit + " " + defaults.fontFamily;

				// insert temp. div which contains the text
				var isOldIE = ($.browser.msie && parseInt($.browser.version, 10) < 9)? true : false,
						$textElem = $("<div />").html("QjÖ").css({
							'font-size': defaults.fontSize + defaults.fontUnit,
							'line-height': (isOldIE ? '150%' : ''),
							'font-family': defaults.fontFamily,
							'position': 'absolute',
							'left': '-9999px',
							'padding': 0
						});

				$((isOldIE ? 'body' : 'html')).append($textElem);

				var lineHeight = $textElem.height() + defaults.correctLineHeight, // get line-height for one text line
						scale = 1,
						newDimensions = calcDimensions(lineHeight, defaults, this[0].ctx),
						newWidth = newDimensions.w,
						newHeight = newDimensions.h,
						measureWidth = newDimensions.measureWidth,
						multiLines = newDimensions.multiLines,
						newInnerWidth = measureWidth,
						newInnerHeight = lineHeight * multiLines.length;

				// remove temp. text element again
				$textElem.remove();
				this[0].ctx.restore();

				return {
					lineHeight: lineHeight,
					scale: scale,
					newDimensions: newDimensions,
					newWidth: newWidth,
					newHeight: newHeight,
					measureWidth: measureWidth,
					multiLines: multiLines,
					newInnerWidth: newInnerWidth,
					newInnerHeight: newInnerHeight,
					defaults: defaults
				};
			},

			calcTextDimensions: function(options){
				this[0].ctx.save();

				var rescaledDimensions = this.rescaleDimensions(options),
						lineHeight = rescaledDimensions.lineHeight,
						scale = rescaledDimensions.scale,
						newDimensions = rescaledDimensions.newDimensions,
						newWidth = newDimensions.w,
						newHeight = newDimensions.h,
						measureWidth = newDimensions.measureWidth,
						multiLines = newDimensions.multiLines,
						newInnerWidth = measureWidth,
						newInnerHeight = lineHeight * multiLines.length,
						defaults = rescaledDimensions.defaults;

				// if multiline text should fit into borders
				if(defaults.maxWidth || defaults.maxHeight){

					if(defaults.resizable){
						var scaleWidth = defaults.maxWidth/newWidth,
							scaleHeight = defaults.maxHeight/newHeight,
							testScale = 1,
							rescaled = false;

						if(defaults.maxWidth && newWidth>defaults.maxWidth){
							testScale = scaleWidth;
						}
						if(defaults.maxHeight && newHeight>defaults.maxHeight){
							testScale = (scaleHeight < scale)? scaleHeight : scale;
						}

						if(testScale*newHeight < defaults.minSize*multiLines.length){
							rescaled = true;
							$.extend(options, {
								fontSize: defaults.fontSize*testScale * (defaults.minSize*multiLines.length)/(testScale*newHeight)
							});

							rescaledDimensions = this.rescaleDimensions(options);
							lineHeight = rescaledDimensions.lineHeight;
							scale = rescaledDimensions.scale;
							newDimensions = rescaledDimensions.newDimensions;
							newWidth = newDimensions.w;
							newHeight = newDimensions.h;
							measureWidth = newDimensions.measureWidth;
							multiLines = newDimensions.multiLines;
							newInnerWidth = measureWidth;
							newInnerHeight = lineHeight * multiLines.length;
							defaults = rescaledDimensions.defaults;
						}
					}

					// if text should be scaled so that it fits into borders (maxWidth & maxHeight)
					if(defaults.resizable && newHeight > defaults.minSize*multiLines.length && !rescaled){
						if(defaults.maxWidth && newWidth>defaults.maxWidth){
							scale = defaults.maxWidth/newWidth;
						}
						if(defaults.maxHeight && newHeight>defaults.maxHeight){
							scale = (defaults.maxHeight/newHeight < scale)? defaults.maxHeight/newHeight : scale;
						}
						defaults.fontSize *= scale;
					// if text should be trimmed so that it fits into borders (maxWidth & maxHeight)
					}else{
						if(defaults.resizeCanvas){

							while(newWidth>=defaults.maxWidth || newHeight>=defaults.maxHeight){
								defaults.text = defaults.text.substr(0, defaults.text.length-1);

								newDimensions = calcDimensions(lineHeight, defaults, this[0].ctx);
								newWidth = newDimensions.w;
								newHeight = newDimensions.h;
							}

							measureWidth = newDimensions.measureWidth;
							multiLines = newDimensions.multiLines;
							newInnerWidth = measureWidth;
							newInnerHeight = lineHeight * multiLines.length;
						}
					}

					newHeight = Math.ceil(scale*newHeight);
					newWidth = Math.ceil(scale*newWidth);
					newInnerHeight = Math.ceil(scale*newInnerHeight);
					newInnerWidth = Math.ceil(scale*newInnerWidth);
				}

				if(defaults.resizeCanvas || defaults.phi !== 0){
					defaults.x = 0.5 * (newWidth - newInnerWidth);
					defaults.y = 0.5 * (newHeight - newInnerHeight);
				}

				this[0].ctx.restore();

				return {
					w: newWidth,
					h: newHeight,
					innerW: newInnerWidth,
					innerH: newInnerHeight,
					measureWidth: measureWidth,
					multiLines: multiLines,
					lineHeight: lineHeight,
					scale: scale,
					text: defaults.text,
					x: defaults.x,
					y: defaults.y,
					fontSize: defaults.fontSize
				};
			},

			// multiline for canvas
			multiline: function(options){
				var defaults = $.extend({
				x: 0,                   // horizontal coordinate
				y: 0,                   // vertical coordinate
				style: 'fillText',      // text method (fillText or strokeText)
				fontSize: 12,
//        resizable: true,
				fontFamily: 'Arial',
				fontUnit: 'pt',
				phi: 0,                 // rotate angle (in degree)
				delimiter: "\n",        // delimter to detect new lines
				align: "left",          // text align (left, right, center)
				rp: 'c',                // rotation coordinates: nw, n, ne, w, c (default), e, sw, s, se
//        maxWidth: undefined,
//        maxHeight: undefined,
//        resizeCanvas: false,
				correctLineHeight: 0,
				text: 'default'
				}, options);

				// execute multiline for every element in the jQuery-query
				return this.each(function(){

					// test text method
					defaults.style = (defaults.style.match(/fillText|strokeText/))? defaults.style : 'fillText';

					// set font
					this.ctx.font = defaults.fontSize + defaults.fontUnit + " " + defaults.fontFamily;

					// insert temp. div which contains the text
					var isOldIE = ($.browser.msie && parseInt($.browser.version, 10) < 9)? true : false,
							$textElem = $("<div />").html("QjÖ").css({
								'font-size': defaults.fontSize + defaults.fontUnit,
								'line-height': (isOldIE ? '150%' : ''),
								'font-family': defaults.fontFamily,
								'position': 'absolute',
								'left': '-9999px',
								'padding': 0
							});

					// append text element to body or html element - depending on used user agent
					$((isOldIE ? 'body' : 'html')).append($textElem);

					// get line-height for one text line
					var lineHeight = $textElem.height() + defaults.correctLineHeight,
							newDimensions = calcDimensions(lineHeight, defaults, this.ctx),
							scale = newDimensions.scale,
							newWidth = newDimensions.w,
							newHeight = newDimensions.h,
							measureWidth = newDimensions.measureWidth,
							multiLines = newDimensions.multiLines,
							multiObj = {};

					// remove temp. text element again
					$textElem.remove();

					// save context
					this.ctx.save();

					// determine the max line width
					for(var i=0, iLen=multiLines.length; i<iLen; i++){
						multiObj[i] = {};
						multiObj[i].width = this.ctx.measureText(multiLines[i]).width;
					}

					// move origin to the start coordinate transfered by the parameters x and y
					this.ctx.translate(defaults.x, defaults.y);

					// set font attributes again
					this.ctx.font = defaults.fontSize + defaults.fontUnit + " " + defaults.fontFamily;
					this.ctx.fontSize = defaults.fontSize;
					this.ctx.fontFamily = defaults.fontFamily;
					this.ctx.fontUnit = defaults.fontUnit;
					this.ctx.textBaseline = 'middle'; // top, hanging, middle, alphabetic, ideographic, bottom

					// calculate rotation coorinates
					var rotPoint = {x: 0, y: 0};
					switch(defaults.rp){
						case 'nw':  // north-west
						rotPoint.x = 0;
						rotPoint.y = 0;
						break;
						case 'n':  // north
						rotPoint.x = measureWidth/2;
						rotPoint.y = 0;
						break;
						case 'ne': // north-east
						rotPoint.x = measureWidth;
						rotPoint.y = 0;
						break;
						case 'w':  // west
						rotPoint.x = 0;
						rotPoint.y = lineHeight*multiLines.length/2;
						break;
						case 'c':  // center
						rotPoint.x = measureWidth/2;
						rotPoint.y = lineHeight*multiLines.length/2;
						break;
						case 'e':  // east
						rotPoint.x = measureWidth;
						rotPoint.y = lineHeight*multiLines.length/2;
						break;
						case 'sw': // south-west
						rotPoint.x = 0;
						rotPoint.y = lineHeight*multiLines.length;
						break;
						case 's':  // south
						rotPoint.x = measureWidth/2;
						rotPoint.y = lineHeight*multiLines.length;
						break;
						case 'se': // south-east
						rotPoint.x = measureWidth;
						rotPoint.y = lineHeight*multiLines.length;
						break;
					}

					// move origin to rotation coordinates
					this.ctx.translate(rotPoint.x, rotPoint.y);

					// for debug only: show rotation coordinates
					//this.ctx.fillRect(0, 0, 3, 3);

					// rotate
					this.ctx.rotate(defaults.phi*Math.PI/180);


					// draw multilines
					for(var j=0, jLen=multiLines.length; j<jLen; j++){
						var leftIndent = (defaults.align==="left")
								? 0-rotPoint.x                                          // align left
								: ((defaults.align==="right")
									? (measureWidth-multiObj[j].width-rotPoint.x)         // align right
									: (measureWidth/2-multiObj[j].width/2-rotPoint.x));   // align center

						// draw line in context
						this.ctx[defaults.style](multiLines[j], leftIndent, (j+0.5)*lineHeight-rotPoint.y);
					}

					// restore context
					this.ctx.restore();

				});
			},

			toDataURL: function(options){
				var defaults = $.extend({
					type: 'image/png',    // default MIME-Type
					index: 0        // index of the canvas-element
					}, options);

				return (this[defaults.index] && (this[defaults.index].toDataURL !== undefined))? this[defaults.index].toDataURL(defaults.type) : undefined;
			}

		});

		// default canvas attributes (getter and setter)
		$.each(['fillStyle', 'strokeStyle', 'lineWidth', 'lineCap', 'lineJoin', 'globalAlpha',
		'globalCompositeOperation', 'shadowOffsetX', 'shadowOffsetY', 'shadowBlur',
		'shadowColor', 'miterLimit', 'font' ], function(i, method){
			fn[method] = function(){
				var args = arguments[0];

				// getter
				if(args === undefined){
					return this[0].ctx[method];
				// setter
				}else{
					return this.each(function(){
						this.ctx[method] = args;
					});
				}
			};
		});

		// default canvas methods
		$.each(['fillRect', 'strokeRect', 'clearRect', 'drawImage', 'fillText', 'strokeText', 'measureText', 'rotate', 'scale', 'translate',
		'save', 'restore', 'createLinearGradient', 'createRadialGradient', 'createPattern', 'getImageData',
		'putImageData', 'arc', 'arcTo', 'beginPath', 'bezierCurveTo', 'clip', 'closePath', 'fill',
		'isPointInPath', 'lineTo', 'moveTo', 'quadraticCurveTo', 'rect', 'stroke'], function(i, method){
			fn[method] = function(){
				var args = arguments;
				return this.each(function(){
					this.ctx[method].apply(this.ctx, args);
				});
			};
		});

		// write canvas plugin in jQuery Prototype
		$.fn.canvas = function(options){
			var result = new qpCanvas();

			this.each(function(){
				result.push(init(this));
			});

			result.prevObject = this;

			return result;
		};
	});

})(jQuery);