use strict;
use Test::More tests => 11;
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Database::API::Connection qw( ConnectTestSite DisconnectSite );
use DE_EPAGES::Object::API::Factory qw ( LoadObjectByPath ExistsObjectByPath LoadRootObject);
use DE_EPAGES::Core::API::Object::DateTimeFormatter;
use DE_EPAGES::Object::API::XML::ObjectImportHandler qw (ImportObjects);
use DE_EPAGES::Product::API::Table::ProductTypeFontColor qw (InsertProductTypeFontColor
    GetInfoProductTypeFontColorsByProductTypeID UpdateProductTypeFontColor DeleteProductTypeFontColor);

## no critic qw( ProhibitMagicNumbers )

my $ROOTPATH = '/Test';

sub TestProductType {

    #CustomizeIsFontSizeSelectable & CustomizeFontSize from XML

    my $ProductType = LoadObjectByPath('/Test/TestShop/ProductTypes/Shirt');

    $ProductType = LoadObjectByPath('/Test/TestShop/ProductTypes/CustomizableProducts');
    ok( $ProductType->get('CustomizeIsFontSizeSelectable'),
        'ProductType.CustomizableProducts.CustomizeIsFontSizeSelectable' );
    is( $ProductType->get('CustomizeFontSize'), 66 , 'ProductType.CustomizableProducts.CustomizeFontSize' );


    #CustomizeIsFontSizeSelectable & CustomizeFontSize
    $ProductType->set({'CustomizeIsFontSizeSelectable' => '0', 'CustomizeFontSize' => '77'},0);
    ok( ! $ProductType->get('CustomizeIsFontSizeSelectable'),
        'ProductType.CustomizableProducts.CustomizeIsFontSizeSelectable' );
    is( $ProductType->get('CustomizeFontSize'), 77 , 'ProductType.CustomizableProducts.CustomizeFontSize' );

    # FontColor
    my $result = GetInfoProductTypeFontColorsByProductTypeID($ProductType->id);
    is(@$result, 0, 'ProductType.CustomizableProducts.Color is empty');

    InsertProductTypeFontColor({'ProductTypeID' => $ProductType->id , 'ColorValue' => 'red', 'Position' => '66'});

    $result = GetInfoProductTypeFontColorsByProductTypeID($ProductType->id);
    is( @$result[0]->{ColorValue} , 'red', 'ProductType.CustomizableProducts.ColorValue');
    is( @$result[0]->{Position} , '66', 'ProductType.CustomizableProducts.ColorPosition');

    UpdateProductTypeFontColor($ProductType->id , 'red', {'Position' => '67'});

    $result = GetInfoProductTypeFontColorsByProductTypeID($ProductType->id);
    is( @$result[0]->{Position} , '67', 'ProductType.CustomizableProducts.ColorPosition updated');

    DeleteProductTypeFontColor($ProductType->id , 'red');
    $result = GetInfoProductTypeFontColorsByProductTypeID($ProductType->id);
    is(@$result, 0, 'ProductType.CustomizableProducts.Color is empty again');
}

sub Cleanup {
    LoadObjectByPath($ROOTPATH)->delete if ExistsObjectByPath($ROOTPATH);
    return 1;
}

sub Init {
    Cleanup();
    LoadRootObject()->insertChild('Object', {'Alias'=>'Test'});
    ImportObjects('import.xml', $ROOTPATH);
    return 1;
}

RunScript(Debug=>1, Sub=> sub {
    ConnectTestSite();
    ok( Init(), 'initialization' );

    TestProductType();

    ok( Cleanup(), 'cleanup');
    DisconnectSite();
});
