#========================================================================================
# package      DE_EPAGES::RemoteSearch::t::IndexStatus
# state        private
#========================================================================================
package DE_EPAGES::RemoteSearch::t::IndexStatus;
use base Test::Class;

use strict;
use warnings;

use Test::More;
use Test::Exception;
use DE_EPAGES::RemoteSearch::API::IndexStatus
  qw ( GetNextState IndexStatusString IndexStatusID INDEXSTATUS_PENDING INDEXSTATUS_COMPLETE INDEXSTATUS_ERROR INDEXSTATUS_UNITIALIZED );
use DE_EPAGES::Core::API::Script qw ( RunScript );
use File::Basename;

sub TestIndexStatusString : Test {
    is( IndexStatusString(0), 'Uninitialized', 'unitialized (string)' );
}

sub TestIndexStatusID : Test {
    is( IndexStatusID(INDEXSTATUS_UNITIALIZED), 0, 'unitialized (id)' );
}

sub TestIndexStatusConst : Test {
    is( INDEXSTATUS_UNITIALIZED, 'Uninitialized', 'const' );
}

sub TestExpectedInvalidStatusID : Test {
    throws_ok { IndexStatusString(-1) } qr/INDEXSTATUS_ID_MISSING/, 'INDEXSTATUS_ID_MISSING';
}

sub TestExpectedInvalidStatusString : Test {
    throws_ok { IndexStatusID('bla') } qr/INDEXSTATUS_STRING_MISSING/, 'INDEXSTATUS_STRING_MISSING';
}

sub TestGetNextStateCompleted : Test {
    is( GetNextState( INDEXSTATUS_PENDING, 'Completed' ), INDEXSTATUS_COMPLETE, 'next state: complete' );
}

sub TestGetNextStateErrors : Test {
    is( GetNextState( INDEXSTATUS_PENDING, 'Error' ), INDEXSTATUS_ERROR, 'next state: errors' );
}

RunScript(
    Sub => sub {
        Test::Class->runtests;
    }
);
