use Test::More;
use strict;
use Text::Diff qw(diff);
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Core::API::File  qw ( GetFileContent WriteFile DeleteFile DeleteDirectory );
use DE_EPAGES::TLE::API::TLE    qw ( ReplaceTLEVars GetByteCodeByText );
use DE_EPAGES::TLE::API::BaseProcessor;
use DE_EPAGES::TLE::t::GUIObjects;
use DE_EPAGES::TLE::t::tlevars;

our $TEST_ONE;

sub Main {
    Initialize();
    if ( defined $TEST_ONE ) {
        plan tests => 1;
        Test_Package($TEST_ONE );
    }
    else {
        my @Files = glob '*_t.txt';
        plan tests => 1 + scalar @Files;

        foreach my $File ( @Files ) {
            if($File =~ /^(.+)_t\.txt$/) {
                my $TestCase = $1;
                Test_Package($TestCase );
            }
            else {
                die "invalid file name $File";
            }
        }

        Test_ReplaceTLEVars( '05_cond' );
    }
    Cleanup();
}

local $| = 1;

RunScript(
    'Sub' => \&Main
);

sub ErrorDefinition {
    return {
        NO_EQUAL_LINE => 'Output is different at line #AtLine'
    };
}

sub Test_ReplaceTLEVars {
    my ($i) = @_;

    my $rTemplate = GetFileContent( $i.'_t.txt' );
    my $byteCode = GetByteCodeByText( $rTemplate );
    my %Vars = (
        %{ DE_EPAGES::TLE::t::tlevars::GetLoops($i) },
        %{ DE_EPAGES::TLE::t::tlevars::GetVars($i) }
    );
    my $isContent = ReplaceTLEVars($byteCode, \%Vars );
    my $exContent = ${ GetFileContent($i.'_e.txt') };


    ok( $isContent eq $exContent, "ReplaceTLEVars $i" )
        or diag( diff \$isContent, \$exContent, { STYLE => 'OldStyle' } );
}

sub Test_Package {
    my ($i) = @_;

    my $Loops =  DE_EPAGES::TLE::t::tlevars::GetLoops($i);
    my $Vars  =  DE_EPAGES::TLE::t::tlevars::GetVars($i);

    local $DE_EPAGES::TLE::API::Execute::TEMPLATE_SOURCE = $i.'_t.txt';
    my $isContent = ProcessFileTemplate($i.'_t.txt', $Vars , $Loops);
    my $exContent = ${GetFileContent($i.'_e.txt')};
    my $equal = $isContent eq $exContent;
    ok( $equal, $i );
    if( $equal ) {
        DeleteFile($i.'_i.txt');
    }
    else {
        diag( diff \$isContent, \$exContent, { STYLE => 'OldStyle' } );
        WriteFile($i.'_i.txt', \$isContent);
    }

    return;
}

sub ProcessFileTemplate {
    my ( $FileName, $hVars, $hLoops ) = @_;

    my $TLEProcessor = DE_EPAGES::TLE::API::BaseProcessor->new;
    DE_EPAGES::TLE::API::PrefixHandler->new->register( $TLEProcessor );

    my $GUIHandler = $TLEProcessor->{'NamedFunctionHandler'}->{'GUI'};
    $GUIHandler->registerGUIObject('DE_EPAGES::TLE::t::GUIObjects::Accordion', 'accordion');
    $GUIHandler->registerGUIObject('DE_EPAGES::TLE::t::GUIObjects::List', 'list');
    $GUIHandler->registerGUIObject('DE_EPAGES::TLE::t::GUIObjects::ListItem', 'listitem');

    my $rTemplate = GetFileContent( $FileName );

    my $byteCode = GetByteCodeByText( $rTemplate );
    my %TempVars = (%$hVars, %$hLoops);

    return $TLEProcessor->replaceTLEs($byteCode, \%TempVars);
}

sub Initialize {
    Cleanup();
}

sub Cleanup {
    DeleteDirectory( 'ctmpl', { recursive => 1 } );
}

1;

