#========================================================================================
# Package: DE_EPAGES::Trigger::API::Object::BatchProcessHandler and
#          DE_EPAGES::Trigger::API::BatchProcessor
#
# Test: Testing basic functions (see BatchProcessor2.t as better documented example)
#
#========================================================================================
use strict;
my $print = 0;    # Verbose output; to get an idea what it does
my $countProcessItem = 0; # counter for test
#########################################################################
#
#   Preparations
#
#########################################################################

## no critic (DE_EPAGES::Style::RequireCommentForExportedFunctions)
package My::Hooks::ProductBatchHooks;

use DE_EPAGES::Trigger::API::BatchProcessor
  qw( RegisterHandler HandleItem GetHandler);

sub OnRegisterDummyImportProductsHandler {

    RegisterHandler( 'DummyImportProducts',
        My::API::Object::ProductBatchHandler->new() );

}

sub OnAfterUpdateProduct($) {
    my ($ProductID) = @_;
    HandleItem(
        'DummyImportProducts',
        'My::API::Object::ProductBatchHandler',
        { ProductID => $ProductID }
    );

    my $Handler =
      GetHandler( 'DummyImportProducts', 'My::API::Object::ProductBatchHandler' );
    print 'add Product '
      . $ProductID
      . ' QueueSize: '
      . $Handler->countItems() . "\n"
      if $print;

}

# Mocking TriggerHook - BatchProcessor register BatchHandler with DE_EPAGES::Trigger::API::Trigger::TriggerHook()
# in the database. This is not part of the test and is mocked.


no warnings qw(redefine); ## no critic (TestingAndDebugging::ProhibitNoWarnings)

sub DE_EPAGES::Trigger::API::BatchProcessor::TriggerHook {
    My::Hooks::ProductBatchHooks::OnRegisterDummyImportProductsHandler();

}

use warnings;

#
# ProductBatchHandler - minimal implementation
#

## no critic (Modules::ProhibitMultiplePackages)
package My::API::Object::ProductBatchHandler;

use parent qw( DE_EPAGES::Trigger::API::Object::BatchProcessHandler );

sub processItem {
    my $self = shift;
    my ($hProduct) = @_;

    $countProcessItem++;

    print 'process Product '
      . $hProduct->{ProductID}
      . ' QueueSize: '
      . $self->countItems() . "\n"
      if $print;
}

#########################################################################
#
#   Begin Tests
#
#########################################################################

package My::UI::BusinessLogic;

use strict;
use Test::More;

use DE_EPAGES::Trigger::API::BatchProcessor qw( RunBatchProcess GetHandler );
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Core::API::Error qw( ExistsError GetError );

plan( tests => 5 );

#
# Use case  - CSV import of products
#

# We emulate a csv import like Product::Scripts::ProductExportImport.pl

sub DummyImportProductList {
    my $Handler;

    # Executes your code, which triggers a hook for adding items for BatchHandler (see DummyImportProduct).
    # After execution of your code BatchHandler->proccessItem() is called automatically

    eval {
        $Handler = GetHandler( 'DummyImportProducts', 'My::Hooks::NotExistingHandler' );
    };
    ok( ExistsError(), 'Works as intended: Throw error for GetHandler(...) on non existing BatchHandler' );

    RunBatchProcess(
        'DummyImportProducts',
        sub {
            print 'start Import' . "\n" if $print;
            $Handler = GetHandler( 'DummyImportProducts',
                'My::API::Object::ProductBatchHandler' );

            ok( $Handler, 'Register BatchHandler' );

            foreach my $ProductID ( 0 .. 9 ) {
                DummyImportProduct($ProductID);
            }

            is( $Handler->countItems(), 10, 'adding 10 items to BatchHandler queue' ); ## no critic (ProhibitMagicNumbers)
            print 'end Import -> start batch processing' . "\n" if $print;
        }
    );

    is( $Handler->countItems(), 0, 'all items consumed and queue cleared' );
    # uses global scalar declared on top
    is($countProcessItem, 10, 'called 10 times processItem'); ## no critic qw(ProhibitMagicNumbers)

}

sub DummyImportProduct {
    my ($ProductID) = @_;

    # do CSV import for current Object

    # trigger ProductBatchHandler -> addItem through Hook
    # in production TriggerHook('OnAfterUpdateProduct') would be used
    My::Hooks::ProductBatchHooks::OnAfterUpdateProduct($ProductID);
    return;
}

RunScript( Sub => \&DummyImportProductList );
